/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifdef ALSA

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <linux/asound.h>
#include <sys/asoundlib.h>
#include "audio.h"

#define OPTIMAL_BLKSIZE 512   /* the minimum (optimal) dsp block size*/

static snd_pcm_t *rhandle=NULL;
static audio_buffer_t rbuffer,wbuffer;
static int blocksize=OPTIMAL_BLKSIZE;
static int bufsize;  /* the wanted size*/
static int total_length,real_length;
static snd_pcm_channel_params_t rsndformat,wsndformat;

int open_dsp_alsa(int rate,int channel,int bits,int qsize)
{
  int err,device,cond;

	err=open("/dev/dsp",O_RDWR|O_NONBLOCK);
	if (err==-1)
	{
		printf("Cannot open audio device because it is busy.\n");
		return(-1);
	} else close(err);

	/* if qsize=0 returns immediately 0. This was just to see if dsp was openable*/
	if (qsize==0)
	{
		close(err);
		return(0);
	}
	
	bufsize=qsize;
  	while(blocksize<qsize) blocksize=blocksize*2;
  	printf("Acceptable blocksize for query (%i) is %i\n",qsize,blocksize);
  	device=snd_defaults_pcm_device();	
  	err=snd_pcm_open(&rhandle,0,device,SND_PCM_OPEN_DUPLEX);
  if (err!=0)
    {
      printf("Error in snd_pcm_open : %i\n",err);
      printf("%s \n",snd_strerror(err));
      return(0);
    }
  bzero(&rsndformat,sizeof(rsndformat));
  rsndformat.channel=SND_PCM_CHANNEL_CAPTURE;
  rsndformat.mode=SND_PCM_MODE_BLOCK;
  rsndformat.format.interleave=1;
  rsndformat.format.format=SND_PCM_SFMT_S16_LE;
  rsndformat.format.rate=rate;
  rsndformat.format.voices=channel;
  rsndformat.start_mode=SND_PCM_START_DATA;
  rsndformat.stop_mode=SND_PCM_STOP_STOP;
  rsndformat.buf.block.frag_size=blocksize;
  rsndformat.buf.block.frags_min=1;

  cond=1;
  	while(cond)
	{
		err=snd_pcm_channel_params(rhandle,&rsndformat);
		/* if you get an error here, it is surely because we requested a too short frag_size, so retry with a greater value*/
		if ((err<0)&&(blocksize<8192))
		{
			blocksize=blocksize*2;
			rsndformat.buf.block.frag_size=blocksize;
			printf("Warning: block size set to %i.\n",blocksize);
		}
		else cond=0;
	}
			
  if (err!=0)
    {
      printf("Error in snd_pcm_channel_params : %i\n",err);
      printf("%s \n",snd_strerror(err));
      return(0);
    }

  bzero(&wsndformat,sizeof(wsndformat));
  wsndformat.channel=SND_PCM_CHANNEL_PLAYBACK;
  wsndformat.mode=SND_PCM_MODE_BLOCK;
  wsndformat.format.interleave=1;
  wsndformat.format.format=SND_PCM_SFMT_S16_LE;
  wsndformat.format.rate=rate;
  wsndformat.format.voices=channel;
  wsndformat.start_mode=SND_PCM_START_FULL;
  wsndformat.stop_mode=SND_PCM_STOP_ROLLOVER;
  wsndformat.buf.block.frag_size=blocksize;
  wsndformat.buf.block.frags_min=1;

  err=snd_pcm_channel_params(rhandle,&wsndformat);
  if (err!=0)
    {
      printf("Error in snd_pcm_channel_params playback : %i\n",err);
      printf("%s \n",snd_strerror(err));
      return(0);
    }

  /* prepare */

  err=snd_pcm_channel_prepare(rhandle,SND_PCM_CHANNEL_CAPTURE);
  if (err!=0)
    {
      printf("Error in snd_pcm_prepare capture: %i\n",err);
      printf("%s \n",snd_strerror(err));
      return(0);
    }

  err=snd_pcm_channel_prepare(rhandle,SND_PCM_CHANNEL_PLAYBACK);
  if (err!=0)
    {
      printf("Error in snd_pcm_prepare playback: %i\n",err);
      printf("%s \n",snd_strerror(err));
      return(0);
    }

	err=snd_pcm_transfer_size(rhandle,SND_PCM_CHANNEL_CAPTURE);
  	printf("Block size for reading  = %i\n",err);

  	err=snd_pcm_transfer_size(rhandle,SND_PCM_CHANNEL_PLAYBACK);
  	printf("Block size for writing  = %i\n",err);

  	if (err>blocksize)
  	{
  		printf("Warning: blocksize set to %i bytes.\n",err);
  		blocksize=err;
  	}

  total_length=blocksize*(N_BUFS+1);
  real_length=blocksize*N_BUFS;

  rbuffer.abuffer=calloc(total_length,sizeof(char));
  rbuffer.rd_ptr=blocksize;
  rbuffer.wr_ptr=blocksize;
  wbuffer.abuffer=calloc(total_length,sizeof(char));
  wbuffer.rd_ptr=blocksize;
  wbuffer.wr_ptr=0;
  return(0);
}


int set_source_alsa(char source)
{
  int i, p, mix_fd;
  
  if (source == 'c')
    p = 1 << SOUND_MIXER_CD;
  if (source == 'l')
    p = 1 << SOUND_MIXER_LINE;
  if (source == 'm')
    p = 1 << SOUND_MIXER_MIC;
  mix_fd = open("/dev/mixer", O_WRONLY, 0);
  i = ioctl(mix_fd, SOUND_MIXER_WRITE_RECSRC, &p);
  close(mix_fd);
  return(0);
}



int set_in_level_alsa(int a)
{
  int p,mix_fd;
	p=(((int)a)<<8 | (int)a);
  mix_fd = open("/dev/mixer", O_RDWR, 0);
  ioctl(mix_fd,MIXER_WRITE(SOUND_MIXER_IGAIN), &p);
  close(mix_fd);
  return(0);
}


int set_out_level_alsa(int a)
{
  int p,mix_fd;

  p=(((int)a)<<8 | (int)a);
  mix_fd = open("/dev/mixer", O_WRONLY, 0);
  ioctl(mix_fd,(MIXER_WRITE(SOUND_MIXER_PCM)), &p);
  close(mix_fd);
  return(0);
}

int close_dsp_alsa()
{
	if (rhandle!=NULL)
	{
  		snd_pcm_close(rhandle);
  		free(rbuffer.abuffer);
  		free(wbuffer.abuffer);
  		blocksize=OPTIMAL_BLKSIZE;
  		rhandle=NULL;
	}
  	return(0);
}

int read_dsp_alsa(int dspfd,char*buffer, int size)
{
  return(snd_pcm_read(rhandle,buffer,size));
}

int write_dsp_alsa(int dspfd,char*buffer, int size)
{
  return(snd_pcm_write(rhandle,buffer,size));
}

char *get_read_alsa_buffer()
{
  char *ret_ptr;
  int bytesread;

  if ((rbuffer.wr_ptr-rbuffer.rd_ptr)<bufsize)
    {
      //fprintf(stderr,"write pointer = %i ",rbuffer.wr_ptr);
      //performs the real read from to the dsp device
      if (rbuffer.wr_ptr>=total_length) /*or >=*/
	{
	  rbuffer.wr_ptr=blocksize; //return at the beginning
	  rbuffer.rd_ptr=rbuffer.rd_ptr-real_length;
	  // and copy data at the beginning
	  memcpy(rbuffer.abuffer,&rbuffer.abuffer[real_length],blocksize);
	}
      bytesread=snd_pcm_read(rhandle,&rbuffer.abuffer[rbuffer.wr_ptr],blocksize);
      if (bytesread!=blocksize) fprintf(stderr,"Only %i bytes read.\n",bytesread);
      rbuffer.wr_ptr+=blocksize;
    }
      // and then returns the buffer
  ret_ptr=&rbuffer.abuffer[rbuffer.rd_ptr];
  //fprintf(stderr,"read pointer = %i \n",rbuffer.rd_ptr);
  rbuffer.rd_ptr+=bufsize;
  
  return(ret_ptr);
}

char *get_write_alsa_buffer()
{
  char *ret_ptr;
  int bytesread;
  if ((wbuffer.wr_ptr-wbuffer.rd_ptr)>=blocksize)/*or >=*/
    {
      //performs the real write to the dsp device
      bytesread=snd_pcm_write(rhandle,&wbuffer.abuffer[wbuffer.rd_ptr],blocksize);
      if (bytesread!=blocksize) fprintf(stderr,"Only %i bytes written.\n",bytesread);
      wbuffer.rd_ptr+=blocksize;
      if (wbuffer.wr_ptr>=real_length)  /*or >=*/
	{
	  wbuffer.rd_ptr=0; //return at the beginning
	  wbuffer.wr_ptr=wbuffer.wr_ptr-real_length;
	  // and copy data at the beginning
	  memcpy(wbuffer.abuffer,&wbuffer.abuffer[real_length],blocksize);
	}
      
    }
  // and then returns the buffer
  ret_ptr=&wbuffer.abuffer[wbuffer.wr_ptr];
  //fprintf(stderr,"read pointer = %i \n",rbuffer.rd_ptr);
  wbuffer.wr_ptr+=bufsize;
  
  return(ret_ptr);
}

#endif
