/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef OSIP_UA_H
#define OSIP_UA_H

#include <stdio.h>
#include <stdint.h>
#include <osip/sip.h>
#include <osip/smsgtypes.h>
#include "OsipCallLeg.h"


#define IP4_SIZE 20

/* the user agent class*/
typedef struct _OsipUA
{
	contact_t *contact;  /* the user agent identity*/
	char ua_ip4addr[IP4_SIZE];
	int ua_port;
	osip_t *config; /* the config structure driving the stack*/
	char *proxy;
	list_t call_list;  /* the list of call-legs managed by the user agent*/
	int max_calls;  /* the number of call of ua will process. If the limit is exceeded, it will answer BUSY HERE*/
	struct _MediaDesc *maudio;     /* a MediaDescriptor for audio*/
	struct _MediaDesc *mvideo;     /* a MediaDescriptor for video*/
	/*signals*/
	int (*invite)(OsipCallLeg *, void*);   /*INVITE*/
	int (*invite_accepted)(OsipCallLeg *, void*);   /*INVITE_ACCEPTED*/    /* called to signal that media sessions can start*/
	int (*bye)(OsipCallLeg *, void*);  /*BYE*/
	int  (*faillure)(OsipCallLeg *, void*);  /*FAILLURE*/
	int  (*informative)(OsipCallLeg *, void*);  /*various informations*/
	FILE *dbg;
} OsipUA;

/* a structure to give parameters to the ua*/
typedef struct _ua_params
{
	char *contact;
	char *ua_ip4addr;
	int ua_port;
	int max_calls;                         /* the number of call of ua will process. If the limit is exceeded, it will answer BUSY HERE*/
	/* session capabilities */
	struct _MediaDesc *maudio;     /* a MediaDescriptor for audio*/    /* can be NULL*/
	struct _MediaDesc *mvideo;     /* a MediaDescriptor for video*/     /*can be NULL*/
	FILE *dbg;
} ua_params_t;

/* create new user agent with given parameters*/
OsipUA * osip_ua_new(ua_params_t *params);

/*set (or change) ua ip address*/
void osip_ua_set_ip4addr(OsipUA *ua,char *ip4addr);

/* set port */
void osip_ua_set_rtpport(OsipUA *ua,int type,int port);

/* clean profiles capabilities */
void osip_ua_clean_profiles(OsipUA *ua, int type);

/* add a profile to the ua capabilities */
void osip_ua_add_profile(OsipUA *ua,int type,int profile);

/*register callbacks to be advertised when an invite arrives, a sent invite is accepted, or a bye is received, or a faillure occured*/
int osip_ua_signal_connect(OsipUA *ua,char *signal,int (*func)(OsipCallLeg *, void*));

/* destroy and free all ressources allocated by user_agent*/
int osip_ua_destroy( OsipUA *ua);

/* change parameters of an existent user agent*/
int osip_ua_change_params(OsipUA *ua, ua_params_t *params);

/* create a new call-leg managed by an existing use agent*/
OsipCallLeg *osip_call_leg_new(OsipUA *ua);

/* accept an invitation for an existing call-leg*/
list_t * osip_call_leg_accept_invite(OsipCallLeg *call_leg);

/* invite somebody */
int osip_call_leg_invite(OsipCallLeg *call_leg, char *callee);

/* get the list media streams with their profile when a call is starting*/
list_t * osip_call_leg_get_streams(OsipCallLeg *call_leg);

/* bye and destroy the call-leg*/
int osip_call_leg_bye(OsipCallLeg *call_leg);

/**********************************************************************************************************************************************************************/
/*																				Private Functions: do not use them in an application																						*/
/***********************************************************************************************************************************************************************/

extern list_t ua_list;


/* find the user agent to which the message is destinated*/
OsipUA * osip_ua_find(sip_t *sipmsg);

/* returns 1 if call is handled by ua*/
int osip_ua_call_leg_exists(OsipUA *ua,OsipCallLeg *call);

#define AUDIO_PROFILES 0
#define VIDEO_PROFILES 1


#endif