/*
  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "utils.h"
#include <osip/sdp.h>


void respond_to_request(osip_t *config, int tid, int code, char *body)
{
   int osiperr;
	char *reason,*name;
	char *apptype=NULL;
	sip_t *response;
	transaction_t *transaction;
	unsigned long int addr;
	struct hostent *host;
	via_t *via;
	sipevent_t *sipevent;
	transaction = trn_findid(config->transactions, tid);

	msg_init(&response,&osiperr);
	reason = parser_getreason(code);

	if (body!=NULL) apptype=sgetcopy("application/sdp");

    if (1==msg_makereply(code,reason,
			 transaction->lastrequest,
			 response,
			 body, /*local_body*/
			 apptype, /*apptype */
			 0,&osiperr))
	{
		/* temp code*/
		/* we must check if last via has x.x.x.x address. If not, we must resolve it*/
		via=(via_t*)list_get(response->vias,response->vias->nb_elt-1);
		if (via==NULL) printf( "Error getting last via!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
		name=via->host;
		if ((int)(addr = inet_addr(name)) == -1)
		{
		 	host=resolv(name);
		 	if (host==NULL)
		 	{
		 		msg_free(response);
		 		sfree(response);
		 		sfree(reason);
		 		sfree(body);
		 		return;
		 	}
		 	else
		 	{
		 		/* replace host name by ip4 addr */
		 		sfree(name);
		 		via->host=sgetcopy(inet_ntoa( ( (struct in_addr*)host->h_addr)[0] ) );
		 	}
		}

		sipevent = evt_new_fromuser(response);
		sipevent->transactionid =  transaction->transactionid;

		fifo_add(transaction->transactionff,sipevent);

#ifdef WITH_4THREADS
		trn_dispatch(transaction);
#endif
	}
	else
	{
		trace(TRACE_LEVEL1,NULL,"<userctrl.c> Could not create response for current status\n");
		sfree(response);
		sfree(reason);
		if (apptype!=NULL)
	  	sfree(apptype);
		return ;
      }
	sfree(reason);
	if (apptype!=NULL)
      sfree(apptype);
}

char *sdp_compose(int sess_id, char *user, char *ip4addr,char *subject, list_t *media_list)
{
	char *sess;
	sdp_t *sdpmsg;
	timed_t *timed;
	mediad_t *media;
	MediaDesc *md;
	char *sdp;
	char *profs,*port;
	int i;
	
	
	sdpmsg=smalloc(sizeof(sdp_t));
	sdp_init(sdpmsg);
	/* v=0*/
	sdpmsg->v = sdp_initv("0");

	/*o= ...*/
	sess = (char *)smalloc(10);
	sprintf(sess,"%i",sess_id);
	sdpmsg->o = sdp_inito(user,
			sess,
			sess,
			"IN",
			"IP4",
			ip4addr);
	
	/*s= subject of converstation*/
	sdpmsg->s = sdp_inits(subject);

	/* c= ....*/	
	timed = (timed_t *)smalloc(sizeof(timed_t));
	timed_init(timed);
	timed->t = sdp_initt("0","0");
	list_add(sdpmsg->timeds,timed,-1);

	sdpmsg->c = sdp_initc("IN","IP4",ip4addr);
	/* m=...*/
	for (i=0;i<media_list->nb_elt;i++)
	{
		media=smalloc(sizeof(mediad_t));
		mediad_init(media);
		md=(MediaDesc*)list_get(media_list,i);
		profs=media_desc_profiles_2char(md);
		port=media_desc_port_2char(md);
		media->m = sdp_initm("audio",port,"RTP/AVP",profs);
		list_add(sdpmsg->mediads,media,-1);
	}
	/* convert to string */
	sdp=sdp_2char(sdpmsg);
	if (sdp==NULL) printf("Error converting sdpmsg to char.\n");
	sdp_free(sdpmsg);
	sfree(sess);
	return(sdp);
}

/* fill default feilds for reply.                    */
/* OUTPUT: sip_t *dest | structure to store response.*/
/* INPUT : sip_t *request | previous request.        */
/* INPUT : int status | status for response.         */
/* INPUT : char *reason | reasonphrase for response. */
/* INPUT : char *body | body for response.           */
/* INPUT : int options | add a tag in to header.     */
/* return -1 on error */
/* TODO: add built-in support for record-route and route */
int
msg_makereply(int status,char *reason,sip_t *request,
	      sip_t *response,char *body,char *content_type
	      ,int options, err_t *err)
{
  int pos;
  int i;

  /* initialise sip_t structure */
  /* yet done... */

  response->strtline = (startline_t *)smalloc(sizeof(startline_t));
  response->strtline->sipversion = (char *)smalloc(8*sizeof(char));
  sprintf(response->strtline->sipversion,"SIP/2.0");
  response->strtline->statuscode = (char *)smalloc(5*sizeof(char));
  sprintf(response->strtline->statuscode,"%i",status);
  response->strtline->reasonphrase = (char *)smalloc(strlen(reason)+1);
  sprintf(response->strtline->reasonphrase,"%s",reason);

  response->strtline->rquri     = NULL;
  response->strtline->sipmethod = NULL;

  /* should add a tag in to feild (for useragent only) */
  /* add a boolean to reqest for it */
  {
    char *tmp;
    i = to_2char(request->to, &tmp, err);
    if (i==-1) return -1;
    i = msg_setto(response, tmp, err);
    sfree(tmp);
    if (i==-1) return -1;
    /*
      if (options==ADD_TAG)
      {
      response->to->tag = (char *)smalloc(12);
      sstrncpy(response->to->tag,"tag=faketag",11);
      }
    */
  }
  {
    char *tmp;
    i = from_2char(request->from, &tmp, err);
    if (i==-1) return -1;
    i = msg_setfrom(response, tmp, err);
    sfree(tmp);
    if (i==-1) return -1;
  }

  /* via headers */
  {
    char *tmp;
    pos = 0;
    while (!list_eol(request->vias,pos))
      {
	via_t *via;
	via = (via_t *)list_get(request->vias,pos);
	i = via_2char(via, &tmp, err);
	if (i==-1) return -1;
	i = msg_setvia(response, tmp, err);
	sfree(tmp);
	if (i==-1) return -1;
	pos++;
      }
  }

  {
    char *tmp;
    i = call_id_2char(request->call_id, &tmp, err);
    if (i==-1) return -1;
    i = msg_setcall_id(response, tmp, err);
    sfree(tmp);
    if (i==-1) return -1;
  }
  {
    char *tmp;
    i = cseq_2char(request->cseq, &tmp, err);
    if (i==-1) return -1;
    i = msg_setcseq(response, tmp, err);
    sfree(tmp);
    if (i==-1) return -1;
  }

   /* add personnal contact information (to be used for next request */
   /* (only for useragent) */
   if (body!=NULL)
     {
       char *size;
       i = msg_setbody(response, body, err);
       if (i==-1) return -1;

       size = (char *) smalloc(6*sizeof(char));
       sprintf(size,"%i",strlen(body));
       i = msg_setcontent_length(response, size, err);
       sfree(size);
       if (i==-1) return -1;

       {
	 char *ct = sgetcopy("content-type");
	 i = msg_setheader(response, ct, content_type, err);
	 sfree(ct);
	 if (i==-1) return -1;
       }
     }
   else
     {
       char *cl = sgetcopy("0");
       i = msg_setcontent_length(response, "0", err);
       sfree(cl);
       if (i==-1) return -1;
     }

   return 1;
}


sip_t * sip_compose_request(OsipCallLeg *call_leg,int msgtype,char *sdpbody)
{
   int osiperr;
	sip_t         *sipmsg;
	char *tmp,*name;
	struct hostent *host;
	unsigned long addr;
	OsipUA *ua= (OsipUA*)call_leg->ua;

	msg_init (&sipmsg,&osiperr);

	/* set to header*/
  {/* mandatory */
    to_2char(call_leg->to,&tmp,&osiperr);
    msg_setto(sipmsg,tmp,&osiperr);
    sfree(tmp);
	}
	/*set from header*/
	{
		from_2char(call_leg->from,&tmp,&osiperr);
    	   msg_setfrom(sipmsg,tmp,&osiperr);
    	   sfree(tmp);
	}
	/* set via header: use ip4addr instead of name*/
	{
    char *via = (char *)smalloc(60*sizeof(char));
    sprintf(via,"SIP/2.0/UDP %s:%i",ua->ua_ip4addr,ua->ua_port);
    msg_setvia(sipmsg,via,&osiperr);
    sfree(via);
  }
	/*set contact header*/
	{
		contact_2char(ua->contact,&tmp,&osiperr);
		msg_setcontact(sipmsg,tmp,&osiperr);
		sfree(tmp);
	}
/*
  if ((*session)->headers!=NULL)
    {
    int pos = 0;
    while (!list_eol((*session)->headers,pos))
      {
      header_t *header;
      header = (header_t *)list_get((*session)->headers,pos);
      sip_setheader(invite,header->hname,header->hvalue);
      pos++;
      }
    }
*/
	/* set first line*/
	sipmsg->strtline = (startline_t *) smalloc(sizeof(startline_t));
	sipmsg->strtline->sipmethod = (char *)smalloc(14);
	switch(msgtype)
	{
		case INVITE:
			strcpy(sipmsg->strtline->sipmethod,"INVITE");
			break;
		case BYE:
			strcpy(sipmsg->strtline->sipmethod,"BYE");
			break;
		case REGISTER:
			strcpy(sipmsg->strtline->sipmethod,"REGISTER");
			break;
		case ACK:
			strcpy(sipmsg->strtline->sipmethod,"ACK");
			break;
	}

  sipmsg->strtline->sipversion = (char *) smalloc(8*sizeof(char));
  sprintf(sipmsg->strtline->sipversion,"SIP/2.0");
  sipmsg->strtline->statuscode   = NULL;
  sipmsg->strtline->reasonphrase = NULL;
  url_init(&sipmsg->strtline->rquri,&osiperr);

	if (ua->proxy==NULL)
  {
		url_2char(sipmsg->to->url,&tmp,&osiperr);
		url_parse(sipmsg->strtline->rquri,tmp,&osiperr);
		sfree(tmp);
	}
	else
	{
		url_parse(sipmsg->strtline->rquri,ua->proxy,&osiperr);
	}
	/* try to resolve the request uri host name if it is not in the x.x.x.x form*/
	name=sipmsg->strtline->rquri->host;
	if ((int)(addr = inet_addr(name)) == -1)
	{
		host=resolv(name);
		if (host==NULL)
		{
			msg_free(sipmsg);
			sfree(sipmsg);
		 	return(NULL);
		}
		else
		{
		 	/* replace host name by ip4 addr */
		 	sfree(name);
		 	sipmsg->strtline->rquri->host=sgetcopy(inet_ntoa(((struct in_addr*)host->h_addr)[0]));
		}
	}

  /* remove unwanted information... They do not appear on request URI */
  /* to do */

	/* set call-id*/
  {
         char *callid;

		call_id_2char(call_leg->callid,&callid,&osiperr);
		msg_setcall_id(sipmsg,callid,&osiperr);
		sfree(callid);
	}
	/* set cseq*/
  {
    char *cseq = (char *)smalloc(30*sizeof(char));
    snprintf(cseq,30,"%i %s",call_leg->cseq_number,sipmsg->strtline->sipmethod);
    msg_setcseq(sipmsg,cseq,&osiperr);
    sfree(cseq);
  }
	/* maybe need to set a subject ...*/
	/* to do*/
	
	/* set application type and sdp body, if given*/
	if (sdpbody!=NULL)
  {
    	char   *ctypev;
    	char size[10];

    	ctypev = sgetcopy("application/sdp\0");
    	msg_setcontent_type(sipmsg,ctypev,&osiperr);

    	sfree(ctypev);
    	
    	msg_setbody(sipmsg,sdpbody,&osiperr);
    	sprintf(size,"%i",strlen(sdpbody));
    	sfree(sdpbody);  /* a voir*/
    	msg_setcontent_length(sipmsg,size,&osiperr);
	}
   else msg_setcontent_length(sipmsg,"0",&osiperr);
	return(sipmsg);
}

int sip_send_request(OsipCallLeg *call_leg,sip_t *sipmsg)
{
	sipevent_t    *sipevent;
	transaction_t *transaction;
   int osiperr;
	OsipUA * ua=(OsipUA*)call_leg->ua;

   if (!MSG_IS_ACK(sipmsg))
   {
		sipevent = evt_new_fromuser (sipmsg);
		transaction = (transaction_t *) smalloc(sizeof(transaction_t));
		trn_init(ua->config,
	   		transaction,
	   		sipmsg->to,
	   		sipmsg->from,
	   		sipmsg->call_id,
	   		sipmsg->cseq,&osiperr);

		call_leg->last_tid=sipevent->transactionid = transaction->transactionid;
		fifo_add(transaction->transactionff,sipevent);
#ifdef WITH_4THREADS
		trn_dispatch(transaction);
#endif
	}
	else
	{
		/*sending an ack is a special case*/
		transaction = trn_findid(ua->config->transactions, call_leg->last_tid);

   		sipevent = evt_new_fromuser(sipmsg);
   		sipevent->transactionid =  transaction->transactionid;

   		fifo_add(transaction->transactionff,sipevent);

#ifdef WITH_4THREADS
    	transaction_handler(transaction);
#endif
	}
	call_leg->last_tid=transaction->transactionid;
	return transaction->transactionid;
}




/* profiles is a list of profiles supported by remote host, and ua_profile is the list for local user agent. The return value is the preferred profile for ua, compatible with
remote host 's list. */
int get_compatible_profile(list_t *ua_profiles,list_t *profiles)
{
	int i,j;
	int local_prof,remote_prof;
	int found=0;
	
	
	if (ua_profiles==NULL) return(-1);
	if (ua_profiles->nb_elt==0) return(-1);
	if (profiles==NULL) return(-1);
	if (profiles->nb_elt==0) return(-1);
	
	for(i=0;((i<ua_profiles->nb_elt) && (!found));i++)
	{
		local_prof=((int*)list_get(ua_profiles,i))[0];
		for (j=0;(j<profiles->nb_elt && !found) ;j++)
		{
			remote_prof=((int*)list_get(profiles,j))[0];
			if (remote_prof==local_prof)
			{
				found=1;
			}
		}
	}
	if (found) return(local_prof);
	else return(-1);
}
			
/*search element el in list, and remove it*/
int list_remove_el(list_t *list,void *el)
{
	int pos=0;
	int cond=0;
	node_t *nd;
	
	nd=list->node;
	while((pos<list->nb_elt)&&(!cond))
	{
		if (nd->element==el) cond=1;
		else
		{
			pos++;
			nd=nd->next;
		}
	}
	if (cond) list_remove(list,pos);
	else return(-1);
	return(0);
}
		
void callid_getcopy(call_id_t	*dest,call_id_t *source)
{
	dest->number=smalloc(strlen(source->number)+1);
	strcpy(dest->number,source->number);
	dest->host=smalloc(strlen(source->host)+1);
	strcpy(dest->host,source->host);
}

void
list_free(list_t *li, void (*destructor)(void *))
{
  int pos = 0;
  void *elem;
  while (!list_eol(li,pos))
    {
     elem = list_get(li,pos);
     list_remove(li,pos);
     if (destructor!=NULL) destructor(elem);
    }
}
