/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef UTILS_H
#define UTILS_H

#include <osip/sip.h>
#include <osip/smsg.h>
#include <osip/port.h>
#include "OsipUA.h"



/* profiles is a list of profiles supported by remote host, and ua_profile is the list for local user agent. The return value is the preferred profile for ua, compatible with
remote host 's list. */
int get_compatible_profile(list_t *ua_profiles,list_t *profiles);


/* respond to a request*/
void respond_to_request(osip_t *config,int tid, int code, char *body);

int msg_makereply(int status,char *reason,sip_t *request,
	      sip_t *response,char *body,char *content_type
	      ,int options, err_t *err);
	
/* returns an sdp message with properties given as argument*/
char *sdp_compose(int sess_id, char *user, char *ip4addr,char *subject, list_t *media_list);

/* returns a sip_t message using properties of call_leg, msgtype={INVITE,BYE,REGISTER}, and body(can be NULL)*/
sip_t * sip_compose_request(OsipCallLeg *call_leg,int msgtype,char *sdpbody);

/* send a sip request*/
int sip_send_request(OsipCallLeg *call_leg,sip_t *sipmsg);

/*search element el in list, and remove it*/
int list_remove_el(list_t *list,void *el);

/* copy callid_t source to dest, by allocating fields inside dest*/
void callid_getcopy(call_id_t	*dest,call_id_t *source);

#define LIST_ELEM_DESTRUCTOR(func)     ((void (*)(void *))func)
/* free a list with all its elements and nodes. destructor() is used ot free elements*/
void list_free(list_t *li, void (*destructor)(void *));
#endif