/***************************************************************************
                          codec.c - the base class for codecs.
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <errno.h>
#include "codec.h"
/* sizes in bytes*/
#define UDP_HDR_SZ 8
#define RTP_HDR_SZ 12
#define IP4_HDR_SZ 20   /*20 is the minimum, but there may be some options*/


/* return 1 if codec can be used with bandwidth, 0 else*/
int codec_is_usable(codec_info_t *codec,double bandwidth)
{
	double codec_band;
	double npacket;
	double packet_size;
	
	if (codec->type==CODEC_AUDIO)
	{
		/* calculate the total bandwdith needed by codec (including headers for rtp, udp, ip)*/		
		/* number of packet per second*/
		npacket=2.0*(double)(codec->spec_info.a_info.rate)/(double)(codec->fr_size);
		packet_size=(double)(codec->dt_size)+UDP_HDR_SZ+RTP_HDR_SZ+IP4_HDR_SZ;
		codec_band=packet_size*8.0*npacket;
	}
	else return -EOPNOTSUPP;
	return(codec_band<bandwidth);
}
	

inline void codec_getinfo(Codec *obj,struct codec_info *info)
{
	obj->_getinfo(obj,info);
}

inline void codec_encode(Codec *obj,char *frame, char *data)
{
	obj->_encode(obj,frame,data);
}

inline void codec_decode(Codec *obj,char *data, char *frame)
{
	obj->_decode(obj,data,frame);
}

inline void codec_destroy(Codec *obj)
{
	obj->_destroy(obj);
}
