/*
applet.c - some utils functions that cannot be set in interface.c.

Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <applet-widget.h>
#include <signal.h>
#include <unistd.h>


#include "gui_utils.h"
#include "callbacks.h"
#include "../audio/audio.h"
#include "LinphoneMain.h"

extern LinphoneMain linpmain;
extern GtkWidget *play_vol;
extern GtkWidget *rec_vol;
extern GtkWidget *entry1,*entry2;
extern GtkWidget *appbar1;
extern GtkWidget *main_window;


void main_window_show()
{
	static char first_time=1;
	if (main_window==NULL)
	{
		main_window = create_app1 ();
		entry1=gtk_object_get_data(GTK_OBJECT(main_window),"entry1");
		entry2=gtk_object_get_data(GTK_OBJECT(main_window),"entry2");
		appbar1=gtk_object_get_data(GTK_OBJECT(main_window),"appbar1");
		if (first_time)
		{
			first_time=0;
			/* restore entries history */
  			gnome_entry_load_history (GNOME_ENTRY (entry2));
			gnome_entry_load_history (GNOME_ENTRY (entry1));
		}
		gtk_widget_show (main_window);
	}
}

/* creates the applet button*/
GtkWidget *create_applet()
{
  GtkWidget *frame;
  GtkWidget *button;
  GtkWidget *vbox;
  GtkWidget *applet;

  applet = applet_widget_new("linphone_applet");

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_widget_show(frame);

  vbox = gtk_vbox_new(FALSE, FALSE);
  gtk_container_add(GTK_CONTAINER(frame), vbox);
  gtk_widget_show(vbox);

  button = gtk_button_new();
	gtk_widget_ref(button);
  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_DEFAULT);
  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, TRUE, 0);
	

  	gtk_widget_show(button);
	applet_widget_add (APPLET_WIDGET (applet), frame);
	gtk_object_set_data_full(GTK_OBJECT(applet),"applet_button",button,(GtkDestroyNotify)gtk_widget_unref);
  	gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
                       GTK_SIGNAL_FUNC(on_applet_clicked), NULL);		
  	gtk_signal_connect(GTK_OBJECT(applet), "change_pixel_size",
                       GTK_SIGNAL_FUNC(applet_change_pixel_size), NULL);		 	
	applet_widget_set_tooltip( APPLET_WIDGET (applet),_("linphone"));
	gtk_widget_show(applet);
	return(applet);
};


/* this  just sets level adjustements for startup*/
void set_levels()
{
	GtkWidget *range;
	if (main_window)
	{
		range=gtk_object_get_data(GTK_OBJECT(main_window),"rec_vol");
		gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(range)),(gfloat)linpmain.sound_conf.rec_lev);
		range=gtk_object_get_data(GTK_OBJECT(main_window),"play_vol");
		gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(range)),(gfloat)linpmain.sound_conf.play_lev);
	}
}

/*try to open dsp. If failed, kill sound daemons if allowed by user, and try agin. If fail again, display the error message. Close it in success*/
int try_open_dsp()
{
	int err1,err2,retval=-1;
	GtkWidget *errmsg;
	
	err1=open_dsp(8000,1,16,0);
	if (err1<0)
	{
		if (linpmain.sound_conf.autokill)
		{
			printf("killing sound daemons.\n");
			kill_sound_daemons();
			sleep(1);   /* wait for sound daemons to be killed...*/
		}
		else
		{
			/* here we sould propose to kill the daemons*/
			/* to do: we required a blocking dialog box here : mutex ??*/
			errmsg=create_messagebox2();
			gtk_widget_show(errmsg);
			return(-1);
		}
		err2=open_dsp(8000,1,16,0);
		if (err2<0)
		{
			errmsg=create_messagebox2();
			gtk_widget_show(errmsg);
			return(-1);
		}
		else retval=0;
	}
	else retval=0;
	
	return(retval);
}
	
		

/*called by SIGUSR1*/
void linphone_wakeup(int signum)
{
	printf("Receiving wake up signal...\n");
	main_window_show();
}


/*called by SIGINT*/
void linphone_exit(int signum)
{
	printf("Exiting gracefully...\n");
	signal(SIGINT,SIG_DFL);
	remove_lock_file();
	applet_widget_gtk_main_quit();
}

/* construct a codec GtkCList*/
void codec_list_construct(GtkCList *clist,GList *codecs)
{
	GList *elem;
	codec_info_t *codec;
	char bitrate[10];
	char *codec_desc[3];
	int row=0;
	
	elem=codecs;
	while(elem!=NULL)
	{
		codec=(codec_info_t*)(elem->data);
		sprintf(bitrate,"%g",((double)codec->bitrate)/1000.0);
		codec_desc[0]=codec->name;
		codec_desc[1]=bitrate;
		//printf("Appending %s\n",codec->name);
		if (codec->is_selected) codec_desc[2]=_("Enabled");
		else codec_desc[2]=_("Disabled");
		gtk_clist_append(clist,codec_desc);
		gtk_clist_set_row_data(clist,row,(void*)codec);
		if (!codec->is_usable) gtk_clist_set_selectable(clist,row,-1);
		
		elem=g_list_next(elem);
		row++;
	}
	if (row) gtk_clist_select_row(clist,0,0);
}

/* get codec_info_t GList from codec GtkCList */
GList *codec_list_retrieve(GtkCList *clist)
{
	int row=0;
	char *text;
	codec_info_t *cod;
	GList *list=NULL;
	
	while(gtk_clist_get_text(clist,row,2,&text)==1)
	{
		cod=gtk_clist_get_row_data(clist,row);
		if (strcmp(text,_("Enabled"))==0) cod->is_selected=1;
		else cod->is_selected=0;
		//free(text);
		list=g_list_append(list,(gpointer)cod);
		row++;
	}
	return(list);
}

/* update graphical (only) status for codecs list*/
void codec_list_update(GtkCList *clist,double bandwidth)
{
	int row=0,u;
	codec_info_t *cod;
	GList *list=clist->row_list;
	GdkColor colors[2]={{0,65535,0,0},{0,0,0,65535}};
	
	while(list!=NULL)
	{
		cod=gtk_clist_get_row_data(clist,row);
		u=codec_is_usable(cod,bandwidth);
		gtk_clist_set_selectable(clist,row,u);
		gtk_clist_set_foreground(clist,row,&colors[u]);
		list=g_list_next(list);
		row++;
	}
}
