/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


/*
 * main.c initial gnr par Glade. Editer ce fichier  votre
 * convenance. Glade n'crira plus dans ce fichier.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <applet-widget.h>
#include <libgnomeui/gnome-window-icon.h>
#include <signal.h>



#include "interface.h"
#include "support.h"
#include "callbacks.h"

#include "LinphoneMain.h"
#include "../osipua/OsipUA.h"
#include "sip_handlers.h"
#include "../audio/ring.h"
#include "../audio/audio.h"
#include "../gsmlib/gsm_wrapper.h"
#include "../lpc10-1.5/lpc10_wrapper.h"
#include "../G711/g711alaw.h"
#include "../G711/g711ulaw.h"

#include "gui_utils.h"


/* some globals objects*/
OsipUA *ua;
GtkWidget *main_window=NULL,*killmsg,*applet;
GtkWidget *entry2,*entry1;
GtkWidget *appbar1;

LinphoneMain linpmain;


/* these are useful messages for the appbar1 */
gchar *ready;
gchar *end;
gchar *contacting;
gchar *connected;
gchar *cancel;
gchar *contacted;


struct poptOption poptCmds[3]=
{
	{
   	 "applet", /*const char * longName;*/ /* may be NULL */
   	 'a', /*char shortName;   */     /* may be '\0' */
   	 POPT_ARG_NONE, /*int argInfo;*/
   	 &linpmain.gui_conf.is_applet, /*void * arg;*/            /* depends on argInfo */
   	 0, /*int val; */              /* 0 means don't return, just update flag */
   	 NULL, /*char * descrip;*/        /* description for autohelp -- may be NULL */
   	 "" /*char * argDescrip; */    /* argument description for autohelp */
   	 },
   	 {
   	 "daemon", /*const char * longName;*/ /* may be NULL */
   	 'd', /*char shortName;   */     /* may be '\0' */
   	 POPT_ARG_NONE, /*int argInfo;*/
   	 &linpmain.gui_conf.is_daemon, /*void * arg;*/            /* depends on argInfo */
   	 0, /*int val; */              /* 0 means don't return, just update flag */
   	 NULL, /*char * descrip;*/        /* description for autohelp -- may be NULL */
   	 "" /*char * argDescrip; */    /* argument description for autohelp */
   	 },
   	 { NULL, 0, 0, NULL, 0 }
};


int
main (int argc, char *argv[])
{
	int err;
	pid_t pid;
	GtkWidget *msgbox1;
	GtkWidget *warninglocal;
	char warning[400];
	poptContext return_ctx;

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif
	g_thread_init(NULL);
	if (!g_thread_supported())
  	{
		printf(_("Threads not supported by glib. Upgrade your glib.\n"));
		return(-1);
	}
	
	poptCmds[0].descrip=_("Run linphone as a gnome-applet.");
	poptCmds[1].descrip=_("Run linphone as a daemon (for use without gnome).");
	
	/* init linphone*/
	linphone_main_init(&linpmain);
	
	applet_widget_init("linphone",VERSION,argc,argv,poptCmds,0,&return_ctx);
	
	gnome_window_icon_set_default_from_file (PACKAGE_DATA_DIR"pixmaps/linphone/linphone2.png");
	
	if ( (linpmain.gui_conf.is_applet) && (linpmain.gui_conf.is_daemon))
	{
		printf("Error: --daemon and --applet are incompatible options.\n");
		return(-1);
	}
	if (linpmain.gui_conf.is_applet) printf("Running linphone as a gnome applet.\n");
	if (linpmain.gui_conf.is_daemon)
	{
		printf("Running linphone as a daemon.\n");
		set_lock_file();
		signal(SIGUSR1,linphone_wakeup);
		signal(SIGINT,linphone_exit);
	}
	if ((!linpmain.gui_conf.is_daemon) && (!linpmain.gui_conf.is_applet))
	{
		/* see if there is not a linphone --daemon running at the moment*/
		pid=get_lock_file();
		if (pid>0)
		{
			kill(pid,SIGUSR1);
			printf("It seems linphone is already running as a daemon (pid=%i). I will wake up it...\n",pid);
			return(0);
		}
	}
		
	/* initialize the mesages for the appbar */
	ready=_("Ready.");
  	end=_("Communication ended.");
	contacting= _("Contacting ");
	contacted=_("is calling you.");
	connected=_("Connected.");
	cancel=_("Call cancelled.");

	
	
	/* register codecs*/
  	linphone_main_register_codec(&linpmain,&gsm_codec_info);
	linphone_main_register_codec(&linpmain,&LPC10codec_info);
	linphone_main_register_codec(&linpmain,&g711alaw_info);
	linphone_main_register_codec(&linpmain,&g711ulaw_info);
	
	/* try to read config from config file*/
	err=linphone_main_read_config(&linpmain);
	if (err==-1)    /* if no config file, set defaults*/
	{
		err=linphone_main_first_startup(&linpmain);
		/* if we have mutiple interface, we must warn the user */
		if (linpmain.net_conf.if_count>1)
		{
			msgbox1=create_messagebox1();
			gtk_widget_show(msgbox1);
		}
	}
	/* warn if previous network interface cannot be found*/
	if (err==-2)
	{
		GtkWidget *wlabel,*ifnotfound;
		sprintf(warning, _("Cannot find network previously used interface %s.\n"
									"If your computer is temporary connected to the internet, please connect and then run linphone.\n"
									"If you want to change your default network interface, go to the parameters 'box."),linpmain.net_conf.if_name);
		ifnotfound=create_ifnotfound();
		wlabel = gtk_label_new (warning);
		gtk_box_pack_start (GTK_BOX(GNOME_DIALOG (ifnotfound)->vbox), wlabel, TRUE, TRUE, 0);
		gtk_widget_show(ifnotfound);
		gtk_widget_show(wlabel);
	}
	else if (err==-3) /* warn if local loopack is used*/
	{
		printf("Warning: local loopback is used.\n");
		warninglocal=create_warninglocal();
		gtk_widget_show(warninglocal);
	}	
	
	if (linpmain.gui_conf.is_applet)
	{
		applet=create_applet();
	}
	
	if ((!linpmain.gui_conf.is_daemon) && (!linpmain.gui_conf.is_applet))
	{
		main_window_show();	
	}
	
  	/* create and start sip user agent */
	linphone_main_start(&linpmain);

  /* open the debug file for rtp*/
  /*linpmain.rtp_conf.rtp_dbg=fopen("/tmp/lprtp.dbg","w");*/
  linpmain.rtp_conf.rtp_dbg=NULL;
	
	if (linpmain.sound_conf.autokill==0)
  	{
		/* check if we have full access to the audio device */
		err=open_dsp(8000,1,16,2048);
		if (err==-1)
		{
			killmsg=create_kill_msg();
			gtk_widget_show(killmsg);
		}else close_dsp();
	}

	
  /***************** and then run the graphical interface */
	set_levels();
	applet_widget_gtk_main ();

  	/* exit properly */
  	
  	close_ringfile();
 	close_ringbackfile();
  	linphone_main_stop(&linpmain);
  	
  	fflush(NULL);
  	if (linpmain.sip_conf.sip_dbg!=NULL) fclose(linpmain.sip_conf.sip_dbg);
  	if (linpmain.rtp_conf.rtp_dbg!=NULL) fclose(linpmain.rtp_conf.rtp_dbg);
  	linphone_main_save_config(&linpmain);
  	remove_lock_file();
	exit(0);
  	return 0;
}



